# -----------------------------------------------------------------------------
# $Id: Cache.pm,v 1.4 2004/02/23 02:46:19 topia Exp $
# -----------------------------------------------------------------------------
package Client::Cache;
use strict;
use warnings;
use base qw(Module);
use Mask;
use Multicast;
use NumericReply;

sub _yesno {
    my ($this, $value, $default) = @_;

    return $default || 0 if (!defined $value);
    return 0 if ($value =~ /[fn]/); # false/no
    return 1 if ($value =~ /[ty]/); # true/yes
    return 1 if ($value); # Ƚ
    return 0;
}

sub message_io_hook {
    my ($this,$msg,$io,$type) = @_;

    if ($io->isa('IrcIO::Server')) {
	if ($type eq 'out' &&
		$msg->command eq 'MODE' &&
		    Multicast::channel_p($msg->param(0)) &&
			    !defined $msg->param(1)) {
	    my $ch = $io->channel($msg->param(0));
	    if (defined $ch) {
		$ch->remark('fetching-switches', 1);
	    }
	} elsif ($type eq 'in' &&
		     $msg->command eq RPL_CHANNELMODEIS &&
			 Multicast::channel_p($msg->param(1))) {
	    my $ch = $io->channel($msg->param(1));
	    if (defined $ch) {
		$ch->remark('fetching-switches', undef, 'delete');
	    }
	} elsif ($type eq 'out' &&
		     $msg->command eq 'WHO' &&
			 Multicast::channel_p($msg->param(0))) {
	    my $ch = $io->channel($msg->param(0));
	    if (defined $ch) {
		$ch->remark('fetching-who', 1);
	    }
	} elsif ($type eq 'in' &&
		     $msg->command eq RPL_WHOREPLY &&
			 Multicast::channel_p($msg->param(1))) {
	    # Թ塢ĤǤⵢäƤǼä
	    my $ch = $io->channel($msg->param(1));
	    if (defined $ch) {
		$ch->remark('fetching-who', undef, 'delete');
	    }
	}
    }
    return $msg;
}

sub message_arrived {
    my ($this,$msg,$sender) = @_;

    # ϤƤ last ȴ
    while (1) {
	# 饤ȤΥå
	last unless ($sender->isa('IrcIO::Client'));
	# ưϵĤƤ뤫?
	last unless ((!defined $sender->option('no-cache')) ||
			 !$this->_yesno($sender->option('no-cache')));
	if ($msg->command eq 'MODE' &&
		$this->_yesno($this->config->use_mode_cache) &&
		    Multicast::channel_p($msg->param(0)) &&
			    !defined $msg->param(1)) {
	    my $chan_long = $msg->param(0);
	    my ($chan_short, $network_name) = Multicast::detach($chan_long);
	    my $network = RunLoop->shared_loop->network($network_name);
	    unless (defined $network) {
		::debug_printmsg(
		    __PACKAGE__.': "'.$network_name.
			'" network is not found in tiarra.'
		       );
		last;
	    }
	    my $ch = $network->channel($chan_short);
	    last if !defined $ch;
	    if ($ch->remark('switches-are-known')) {
		my $remark = $sender->remark('mode-cache-used') || {};
		if (!exists $remark->{$chan_long}) {
		    $sender->send_message(
			IRCMessage->new(
			    Prefix => RunLoop->shared_loop->sysmsg_prefix('system'),
			    Command => RPL_CHANNELMODEIS,
			    Params => [
				RunLoop->shared_loop->current_nick,
				$chan_long,
				$ch->mode_string,
			       ],
			    Remarks => {
				'fill-prefix-when-sending-to-client' => 1,
			    },
			   )
		       );
		    $remark->{$chan_long} = 1;
		    $sender->remark('mode-cache-used', $remark);
		    return undef;
		}
	    } else {
		if ($ch->remark('fetching-switches')) {
		    # Ƥ륯饤Ȥʤ顢Ͼä
		    return undef;
		}
		# ˤäƤ餦
	    }
	} elsif ($msg->command eq 'WHO' &&
		     $this->_yesno($this->config->use_who_cache) &&
			 Multicast::channel_p($msg->param(0))) {
	    my $chan_long = $msg->param(0);
	    my ($chan_short, $network_name) = Multicast::detach($chan_long);
	    my $network = RunLoop->shared_loop->network($network_name);
	    unless (defined $network) {
		::debug_printmsg(
		    __PACKAGE__.': "'.$network_name.
			'" network is not found in tiarra.'
		       );
		last;
	    }
	    my $ch = $network->channel($chan_short);
	    last unless (defined $ch);
	    my $remark = $sender->remark('who-cache-used') || {};
	    if (!exists $remark->{$chan_long}) {
		# cache äƤ뤫狼ʤᡢ
		# ȤꤢäƤߤơ­ʤä餢롣
		my $message_tmpl = IRCMessage->new(
		    Prefix => RunLoop->shared_loop->sysmsg_prefix('system'),
		    Command => RPL_WHOREPLY,
		    Params => [
			RunLoop->shared_loop->current_nick,
			$chan_long,
		       ],
		    Remarks => {
			'fill-prefix-when-sending-to-client' => 1,
		    },
		   );
		my @messages;
		eval {
		    foreach (values %{$ch->names}) {
			my $p_ch = $_;
			my $p = $p_ch->person;

			# ƽפǤʤ塢
			# ¤ñʥǡ¤ޤ
			# դƤ
			if (!$p->username || !$p->userhost ||
				!$p->realname || !$p->server) {
			    # ǡ­롣
			    die 'cache data not enough';
			}

			my $message = $message_tmpl->clone(deep => 1);
			$message->param(2, $p->username);
			$message->param(3, $p->userhost);
			$message->param(4, $p->server);
			$message->param(5,
					Multicast::global_to_local($p->nick,
								   $network));
			$message->param(6,
					(length($p->away) ? 'G' : 'H') .
					    $p_ch->priv_symbol);
			$message->param(7,
					$network->remark('server-hops')
					    ->{$p->server}.' '.
						$p->realname);
			push(@messages, $message);
		    }
		};
		if (!$@) {
		    my $message = $message_tmpl->clone(deep => 1);
		    $message->command(RPL_ENDOFWHO);
		    $message->param(2, 'End of WHO list.');
		    push(@messages, $message);
		    map {
			$sender->send_message($_);
		    } @messages;
		    $remark->{$chan_long} = 1;
		    $sender->remark('who-cache-used', $remark);
		    return undef;
		} else {
		    if ($ch->remark('fetching-who')) {
			# Ƥ륯饤Ȥʤ顢Ͼäؾ衣
			return undef;
		    }
		    # ˤäƤ餦
		}
	    }
	}
	last;
    }

    return $msg;
}

1;
=pod
info: ǡ򥭥å夷ƥФ䤤碌ʤ褦ˤ
default: off

# åѤƤ⡢ȤΤ³ǽΰ٤Ǥ
# ܤ̤̾˥Ф䤤碌ޤ
# ޤ饤ȥץ no-cache ꤹưޤ

# mode åѤ뤫
use-mode-cache: 1

# who åѤ뤫
use-who-cache: 1
=cut
